;****************************************************************************
; Program	: Interrupt 01
; Description	: Interrupt to set LED Running
;		: Interrupt 0 , S1 to Set LED running left only
;		: Interrupt 1 , S2 to Set LED running right only
; For		: MCS-51 Microcontroller Training System
; Filename	: lab2301.asm
; Environment	: ASEM-51 Macro assembler
; Copyright (C) 2002-2005 Innovative Experiment Co.,Ltd.
;****************************************************************************

;----------------------------------------------------------------------------
; Define Port&Pin Name
;----------------------------------------------------------------------------
DRIVER_LE	BIT	P1.4		; Driver Latch Enable (Active High : Level)

INT0_PIN	BIT	P3.2		; Interrupt 0 PIN S2
INT1_PIN	BIT	P3.3		; Interrupt 1 PIN S3
LCD_EN		BIT	P3.6		; LCD Module Enable (Active High : Level)
LCD_RS		BIT	P3.7		; LCD Module Register Select

;----------------------------------------------------------------------------
; Define User Register
;----------------------------------------------------------------------------
LCD_ADDR	EQU	030H		; For keep LCD Address
LCD_DATA	EQU	031H		; For keep LCD Data
LED_STATE	EQU	032H		; For keep LED Running State

;----------------------------------------------------------------------------
; Main Program.
;----------------------------------------------------------------------------

		ORG	0000H		; Reset Vector
		AJMP	INITIAL		; Jump to initial

		ORG	0003H		; IE0 Vector
		AJMP	LED_LEFT	; Jump to Set LED running left

		ORG	000BH		; TF0 Vector
		RETI			; Return interrupt

		ORG	0013H		; IE1 Vector
		AJMP	LED_RIGHT	; Jump to Set LED running right

		ORG	001BH		; TF1 Vector
		RETI			; Return interrupt

		ORG	0023H		; TI+RI Vector
		RETI			; Return interrupt

		ORG	002BH		; TF2+EXF2 Vector
		RETI			; Return interrupt

INITIAL:	MOV	P0,#00000000B	; Clear Databus
		CLR	P1.4		; Clear Latch Enable
		MOV	P3,#00011111B	; Clear status LCD, P3.2-3

MAIN:		ACALL	INIT_LCD	; Call LCD Initial subroutine

		MOV	LED_STATE,#0 	; Initial LED Running State

		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#TITLE_1	; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-07H,40H-47H(Increase automatic)

		ACALL	DELAY_1s	; Delay
		ACALL	DELAY_1s	;

		MOV	IE,#10000101B	; En. EA,EX1,EX0
		CLR	F0		; Clear Flag 0

MAIN_LOOP:	JB	F0,MAIN_SKIP    ; Check Flag 0
		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_LED_RUN ; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD		; 00H-07H,40H-47H(Increase automatic)
		SETB	F0

MAIN_SKIP:	ACALL	RUN_LED			; Set LED ad Shift State
		AJMP	MAIN_LOOP		; Jump to main loop

;----------------------------------------------------------------------------
; Run LED and Shift State
;----------------------------------------------------------------------------
RUN_LED:	MOV	A,LED_STATE	; Get State
		MOV	DPTR,#MASK_LED	; Get Mask data			
		MOVC	A,@A+DPTR	;
		MOV	P0,A		; Move to DATABUS
		ACALL	DRIVE_LE_CLK	; Latch
		ACALL	DELAY_100ms	; Delay
		ACALL	DELAY_100ms	;

		MOV	A,LED_STATE	; Get State
		INC	A		; Mask state in range 0 to 5
		MOV	B,#6		; ( divide by 6 )
		DIV	AB		;
		MOV	LED_STATE,B	; Save remainder as new State

		RET			; Return

;----------------------------------------------------------------------------
; Drive Latch Enable Clock
;----------------------------------------------------------------------------
DRIVE_LE_CLK:	SETB	DRIVER_LE	; Driver Latch Enable
		NOP			; NOP
		CLR	DRIVER_LE	; Driver Latch Disable
		NOP
		RET

;----------------------------------------------------------------------------
; LED LEFT only interrupt subroutine
;----------------------------------------------------------------------------
LED_LEFT:	PUSH	ACC		; Keep Main ACC
		CLR	IP.0		; Set EX0 to lower priority
		SETB	IP.2		; Set EX1 to higher priority

		MOV	A,#00000001B	; Set data to show LED
		MOV	R2,#16		; Set loop as 16 times

LED_LEFT_LOOP:	PUSH	ACC		; Keep Sub. ACC
		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_INT_0 ; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-07H,40H-47H(Increase automatic)
		POP	ACC		; Restore Sub. ACC

		MOV	P0,A		; Move data to LED
		ACALL	DRIVE_LE_CLK	; Latch

		RL	A		; Rotate data to left
		ACALL	DELAY_100ms	; Delay
		ACALL	DELAY_100ms	;
		DJNZ	R2,LED_LEFT_LOOP ; Do loop
		POP	ACC		; Restore Main ACC

		MOV	IP,#0		; Clear all interrupt to lower priority
		CLR	F0		; Clear Flag 0
		RETI

;----------------------------------------------------------------------------
; LED RIGHT only interrupt subroutine
;----------------------------------------------------------------------------
LED_RIGHT:	PUSH	ACC		; Keep Main ACC
		SETB	IP.0		; Set EX0 to higher priority
		CLR	IP.2		; Set EX1 to lower priority

		MOV	A,#10000000B	; Set data to show LED
		MOV	R3,#16		; Set loop as 16 times

LED_RIGHT_LOOP:	PUSH	ACC		; Keep Sub. ACC
		MOV	LCD_ADDR,#000H	; Set Address 00H
		ACALL	SET_ADDR_LCD	;
		MOV	DPTR,#SCR_INT_1 ; Index Pointer ROM to Show LCD
		ACALL	WRLINE_LCD	; 00H-07H,40H-47H(Increase automatic)
		POP	ACC		; Restore Sub. ACC

		MOV	P0,A		; Move data to LED
		ACALL	DRIVE_LE_CLK	; Latch

		RR	A		; Rotate data to right
		ACALL	DELAY_100ms	; Delay
		ACALL	DELAY_100ms	;
		DJNZ	R3,LED_RIGHT_LOOP ; Do loop
		POP	ACC		; Restore Main ACC

		MOV	IP,#0		; Clear all interrupt to lower priority
		CLR	F0		; Clear Flag 0
		RETI

;----------------------------------------------------------------------------
; Define LCD's command constant
;----------------------------------------------------------------------------
LCD_CMD_CLR	EQU	00000001B	; Clear
LCD_CMD_HOME	EQU	00000010B	; Cursor home
LCD_CMD_AUTOINC	EQU	00000110B	; Auto increment
LCD_CMD_OFF	EQU	00001000B	; Off
LCD_CMD_ON	EQU	00001100B	; On
LCD_CMD_BLINK	EQU	00001111B	; Cursor on & blink
LCD_CMD_LSHF	EQU	00011000B	; Shift display to left
LCD_CMD_RSHF	EQU	00011100B	; Shift display to right


;----------------------------------------------------------------------------
; LCD Initialize
;----------------------------------------------------------------------------
INIT_LCD:	ACALL   DELAY_100ms	; Delay

		CLR	LCD_RS		; Clear LCD_RS pin
		MOV	P0,#00111000B	; Force 8bit mode on first times
		ACALL	LCD_CLK		; Pulse LCD clock
		ACALL	DELAY_10ms	; Delay

	; force previous possible mode (4 or 8 bit) to be 8 bit
		MOV	P0,#00111000B	; Force 8bit mode again
		ACALL	LCD_CLK		; Pulse LCD clock

		ACALL	LCD_OFF		; Display off
		ACALL	LCD_AUTOINC	; Set auto increment cursor after write
		ACALL	LCD_HOME	; Return cursor to home
		ACALL	LCD_CLR		; Display cleared
		ACALL	LCD_ON		; Display on
		RET

;----------------------------------------------------------------------------
; LCD Clear Display
;----------------------------------------------------------------------------
LCD_CLR:	MOV	LCD_DATA,#LCD_CMD_CLR	; Display cleared
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_HOME:	MOV	LCD_DATA,#LCD_CMD_HOME	; Return cursor to uppper left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Return Home
;----------------------------------------------------------------------------
LCD_AUTOINC:	MOV	LCD_DATA,#LCD_CMD_AUTOINC; Cursor auto increment after write
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display Off
;----------------------------------------------------------------------------
LCD_OFF:	MOV	LCD_DATA,#LCD_CMD_OFF	; Display off
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Display On
;----------------------------------------------------------------------------
LCD_ON:		MOV	LCD_DATA,#LCD_CMD_ON	; Display on
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Cursor On
;----------------------------------------------------------------------------
LCD_BLINK:	MOV	LCD_DATA,#LCD_CMD_BLINK	; Display blink
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Left Shift Display
;----------------------------------------------------------------------------
LCD_LSHF:	MOV	LCD_DATA,#LCD_CMD_LSHF	; Shift display to left
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; LCD Right Shift Display
;----------------------------------------------------------------------------
LCD_RSHF:	MOV	LCD_DATA,#LCD_CMD_RSHF	; Shift display to right
		AJMP	WRCMD_LCD	; Send command

;----------------------------------------------------------------------------
; Set LCD Address
; I/P:          LCD_ADDR
;----------------------------------------------------------------------------
SET_ADDR_LCD:	CLR     LCD_RS		; Clear LCD_RS pin
		MOV     P0,LCD_ADDR	; Move LCD_ADDR to P0
		ORL	P0,#10000000B	; Set MSB of P0
		ACALL   LCD_CLK		; Pulse LCD clock
		RET			; Return

;----------------------------------------------------------------------------
; LCD Clear Display
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCMD_LCD:	CLR	LCD_RS		; Clear LCD_RS Pin
		MOV	P0,LCD_DATA		; Write command to LCD
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET			; Return

;----------------------------------------------------------------------------
; Write Character to show LCD
; I/P:          LCD_DATA
;----------------------------------------------------------------------------
WRCHAR_LCD:	SETB	LCD_RS		; Set LCD_RS Pin
		MOV	P0,LCD_DATA	; Move LCD_DATA to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		RET						; Return

;----------------------------------------------------------------------------
; Put string from ROM (string must ended with 000H)
; I/P:          DPTR : Locate ROM Address
;----------------------------------------------------------------------------
STRING_LCD:	PUSH	ACC		; Put ACC. into stack
STRING_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		CJNE	A,#0,STRING_LCD_2	; Do until null terminate detect
		POP	ACC		; Pop ACC from stack
		RET			; Return
				
STRING_LCD_2:	MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		AJMP	STRING_LCD_1

;----------------------------------------------------------------------------
; Write Line of 16 Character from ROM
; I/P:		DPTR : Locate ROM Address
;----------------------------------------------------------------------------
WRLINE_LCD:	MOV	R0,#0		; Clear loop counter
WRLINE_LCD_1:	SETB	LCD_RS		; Set LCD_RS Pin
		CLR	A		; Clear ACC.
		MOVC	A,@A+DPTR	; Move data from @DPTR to ACC.
		MOV	P0,A		; Move ACC. to DATABUS
		ACALL	LCD_CLK		; Pulse LCD Clock
		INC	DPTR		; Increase Pointer
		INC	R0		; Increase loop counter
		CJNE	R0,#16,WRLINE_LCD_1 ; Do until 16 times
		RET

;----------------------------------------------------------------------------
; LCD Clk
;----------------------------------------------------------------------------
LCD_CLK:	SETB	LCD_EN		; Pulse Clock to LCD_EN
		ACALL	LCD_DELAY
		CLR	LCD_EN
		ACALL	LCD_DELAY
		RET

;----------------------------------------------------------------------------
; Dummy Delay time LCD_DELAY, 10m, 100m, 1s
;----------------------------------------------------------------------------
LCD_DELAY:	MOV	R7,#002		; Do 2 times
LCD_DELAY_1:	MOV	R6,#0E6H	; Each loop = 1 ms
LCD_DELAY_2:	NOP
		NOP
		DJNZ	R6,LCD_DELAY_2
		DJNZ	R7,LCD_DELAY_1
		RET

DELAY_10ms:	MOV	R7,#010		; Do 10 times
DELAY_10ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_10ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_10ms_2
		DJNZ	R7,DELAY_10ms_1
		RET

DELAY_100ms:	MOV	R7,#100		; Do 100 times
DELAY_100ms_1:	MOV	R6,#0E6H	; Each loop = 1 ms
DELAY_100ms_2:	NOP
		NOP
		DJNZ	R6,DELAY_100ms_2
		DJNZ	R7,DELAY_100ms_1
		RET

DELAY_1s:	MOV	R5,#100		; Do 100 times
DELAY_1s_1:	ACALL	DELAY_10ms
		DJNZ	R5,DELAY_1s_1
		RET

;----------------------------------------------------------------------------
;Define Constant < Store in Flash EEPROM Program Memory >
;----------------------------------------------------------------------------
;			 0123456789ABCDEF
TITLE_1:	DB	' Interrupt DEMO '

SCR_LED_RUN:	DB	' LED Running... '
SCR_INT_0:	DB	' INT0  Activate '
SCR_INT_1:	DB	' INT1  Activate '

MASK_LED:	DB	00011000B
		DB	00100100B
		DB	01000010B
		DB	10000001B
		DB	01000010B
		DB	00100100B

		END